//------------------------------------------------------------------------------
// CERN BE-CO-HT
// Mock Turtle
// https://gitlab.cern.ch/coht/mockturtle
//------------------------------------------------------------------------------
//
// unit name:   main
//
// description: A SystemVerilog testbench for the supplied SVEC MT demo.
//
//------------------------------------------------------------------------------
// Copyright CERN 2018
//------------------------------------------------------------------------------
// Copyright and related rights are licensed under the Solderpad Hardware
// License, Version 2.0 (the "License"); you may not use this file except
// in compliance with the License. You may obtain a copy of the License at
// http://solderpad.org/licenses/SHL-2.0.
// Unless required by applicable law or agreed to in writing, software,
// hardware and materials distributed under this License is distributed on an
// "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
// or implied. See the License for the specific language governing permissions
// and limitations under the License.
//------------------------------------------------------------------------------

`include "mock_turtle_driver.svh"
`include "vhd_wishbone_master.svh"

`timescale 1ns/1ps

module main;

   reg rst_n = 0;
   reg clk_125m = 0;

   always #4ns clk_125m <= ~clk_125m;

   initial begin
      repeat(20) @(posedge clk_125m);
      rst_n = 1;
   end

   // the Device Under Test
   svec_mt_demo #
     (
      .g_sim_bypass_vme(1)
      )
   DUT
     (
      .rst_a_n_i           (rst_n),
      .clk_125m_pllref_p_i (clk_125m),
      .clk_125m_pllref_n_i (~clk_125m),
      .fp_led_line_oen_o   (),
      .fp_led_line_o       (),
      .fp_led_column_o     (),
      .fp_gpio1_a2b_o      (),
      .fp_gpio2_a2b_o      (),
      .fp_gpio34_a2b_o     (),
      .fp_gpio_b           (),
      .sim_wb_i            (Host.out),
      .sim_wb_o            (Host.in),
      .VME_AS_n_i          (),
      .VME_RST_n_i         (),
      .VME_WRITE_n_i       (),
      .VME_AM_i            (),
      .VME_DS_n_i          (),
      .VME_GA_i            (),
      .VME_BERR_b          (),
      .VME_DTACK_n_b       (),
      .VME_RETRY_n_o       (),
      .VME_RETRY_OE_o      (),
      .VME_LWORD_n_b       (),
      .VME_ADDR_b          (),
      .VME_DATA_b          (),
      .VME_BBSY_n_i        (),
      .VME_IRQ_n_o         (),
      .VME_IACK_n_i        (),
      .VME_IACKIN_n_i      (),
      .VME_IACKOUT_n_o     (),
      .VME_DTACK_OE_b      (),
      .VME_DATA_DIR_b      (),
      .VME_DATA_OE_N_b     (),
      .VME_ADDR_DIR_b      (),
      .VME_ADDR_OE_N_b     ());


   IVHDWishboneMaster Host
     (
      .clk_i   (DUT.clk_sys),
      .rst_n_i (DUT.rst_n_sys));

   IMockTurtleIRQ IrqMonitor (`MT_ATTACH_IRQ(DUT.U_Mock_Turtle));

   string fw = "../../../demos/hello_world/firmware/fw-01/fw-hello.bin";

   const uint64_t mt_base = 'h2_0000;

   MockTurtleDriver drv;

   initial begin

      $timeformat (-6, 3, "us", 10);

      @(posedge DUT.rst_n_sys);
      @(posedge DUT.clk_sys);

      drv = new (Host.get_accessor(), mt_base, IrqMonitor);

      drv.init();

      drv.enable_console_irq (0, 1);

      drv.load_firmware(0, fw, 1'b0);

      drv.reset_core(0, 0);

      forever begin
	 drv.update ();
	 #1us;
      end

   end // initial begin

endmodule // main
